@@ -34,6 +34,7 @@ type Options struct {
 
 	MinifyWhitespace  bool
 	ASCIIOnly         bool
+	SourceMap         config.SourceMap
 	AddSourceMappings bool
 	LegalComments     config.LegalComments
 }
@@ -53,11 +54,17 @@ func Print(tree css_ast.AST, options Options) PrintResult {
 	for _, rule := range tree.Rules {
 		p.printRule(rule, 0, false)
 	}
-	return PrintResult{
+	result := PrintResult{
 		CSS:                    p.css,
 		ExtractedLegalComments: p.extractedLegalComments,
-		SourceMapChunk:         p.builder.GenerateChunk(p.css),
 	}
+	if options.SourceMap != config.SourceMapNone {
+		// This is expensive. Only do this if it's necessary. For example, skipping
+		// this if it's not needed sped up end-to-end parsing and printing of a
+		// large CSS file from 66ms to 52ms (around 25% faster).
+		result.SourceMapChunk = p.builder.GenerateChunk(p.css)
+	}
+	return result
 }
 
 func (p *printer) printRule(rule css_ast.Rule, indent int32, omitTrailingSemicolon bool) {
@@ -543,12 +550,18 @@ func (p *printer) printWithEscape(c rune, escape escapeKind, remainingText strin
 	}
 }
 
+// Note: This function is hot in profiles
 func (p *printer) printQuotedWithQuote(text string, quote byte) {
 	if quote != quoteForURL {
 		p.css = append(p.css, quote)
 	}
 
-	for i, c := range text {
+	n := len(text)
+	i := 0
+	runStart := 0
+
+	for i < n {
+		c, width := utf8.DecodeRuneInString(text[i:])
 		escape := escapeNone
 
 		switch c {
@@ -577,7 +590,18 @@ func (p *printer) printQuotedWithQuote(text string, quote byte) {
 			}
 		}
 
-		p.printWithEscape(c, escape, text[i:], false)
+		if escape != escapeNone {
+			if runStart < i {
+				p.css = append(p.css, text[runStart:i]...)
+			}
+			p.printWithEscape(c, escape, text[i:], false)
+			runStart = i + width
+		}
+		i += width
+	}
+
+	if runStart < n {
+		p.css = append(p.css, text[runStart:]...)
 	}
 
 	if quote != quoteForURL {
@@ -591,6 +615,7 @@ const (
 	identNormal identMode = iota
 	identHash
 	identDimensionUnit
+	identDimensionUnitAfterExponent
 )
 
 type trailingWhitespace uint8
@@ -600,7 +625,52 @@ const (
 	canDiscardWhitespaceAfter
 )
 
+// Note: This function is hot in profiles
 func (p *printer) printIdent(text string, mode identMode, whitespace trailingWhitespace) {
+	n := len(text)
+
+	// Special escape behavior for the first character
+	initialEscape := escapeNone
+	switch mode {
+	case identNormal:
+		if !css_lexer.WouldStartIdentifierWithoutEscapes(text) {
+			initialEscape = escapeBackslash
+		}
+	case identDimensionUnit, identDimensionUnitAfterExponent:
+		if !css_lexer.WouldStartIdentifierWithoutEscapes(text) {
+			initialEscape = escapeBackslash
+		} else if n > 0 {
+			if c := text[0]; c >= '0' && c <= '9' {
+				// Unit: "2x"
+				initialEscape = escapeHex
+			} else if (c == 'e' || c == 'E') && mode != identDimensionUnitAfterExponent {
+				if n >= 2 && text[1] >= '0' && text[1] <= '9' {
+					// Unit: "e2x"
+					initialEscape = escapeHex
+				} else if n >= 3 && text[1] == '-' && text[2] >= '0' && text[2] <= '9' {
+					// Unit: "e-2x"
+					initialEscape = escapeHex
+				}
+			}
+		}
+	}
+
+	// Fast path: the identifier does not need to be escaped. This fast path is
+	// important for performance. For example, doing this sped up end-to-end
+	// parsing and printing of a large CSS file from 84ms to 66ms (around 25%
+	// faster).
+	if initialEscape == escapeNone {
+		for i := 0; i < n; i++ {
+			if c := text[i]; c >= 0x80 || !css_lexer.IsNameContinue(rune(c)) {
+				goto slowPath
+			}
+		}
+		p.css = append(p.css, text...)
+		return
+	slowPath:
+	}
+
+	// Slow path: the identifier needs to be escaped
 	for i, c := range text {
 		escape := escapeNone
 
@@ -616,36 +686,15 @@ func (p *printer) printIdent(text string, mode identMode, whitespace trailingWhi
 			}
 
 			// Special escape behavior for the first character
-			if i == 0 {
-				switch mode {
-				case identNormal:
-					if !css_lexer.WouldStartIdentifierWithoutEscapes(text) {
-						escape = escapeBackslash
-					}
-
-				case identDimensionUnit:
-					if !css_lexer.WouldStartIdentifierWithoutEscapes(text) {
-						escape = escapeBackslash
-					} else if c >= '0' && c <= '9' {
-						// Unit: "2x"
-						escape = escapeHex
-					} else if c == 'e' || c == 'E' {
-						if len(text) >= 2 && text[1] >= '0' && text[1] <= '9' {
-							// Unit: "e2x"
-							escape = escapeBackslash
-						} else if len(text) >= 3 && text[1] == '-' && text[2] >= '0' && text[2] <= '9' {
-							// Unit: "e-2x"
-							escape = escapeBackslash
-						}
-					}
-				}
+			if i == 0 && initialEscape != escapeNone {
+				escape = initialEscape
 			}
 		}
 
 		// If the last character is a hexadecimal escape, print a space afterwards
 		// for the escape sequence to consume. That way we're sure it won't
 		// accidentally consume a semantically significant space afterward.
-		mayNeedWhitespaceAfter := whitespace == mayNeedWhitespaceAfter && escape != escapeNone && i+utf8.RuneLen(c) == len(text)
+		mayNeedWhitespaceAfter := whitespace == mayNeedWhitespaceAfter && escape != escapeNone && i+utf8.RuneLen(c) == n
 		p.printWithEscape(c, escape, text[i:], mayNeedWhitespaceAfter)
 	}
 }
@@ -706,8 +755,13 @@ func (p *printer) printTokens(tokens []css_ast.Token, opts printTokensOpts) bool
 			p.print("(")
 
 		case css_lexer.TDimension:
-			p.print(t.DimensionValue())
-			p.printIdent(t.DimensionUnit(), identDimensionUnit, whitespace)
+			value := t.DimensionValue()
+			p.print(value)
+			mode := identDimensionUnit
+			if strings.ContainsAny(value, "eE") {
+				mode = identDimensionUnitAfterExponent
+			}
+			p.printIdent(t.DimensionUnit(), mode, whitespace)
 
 		case css_lexer.TAtKeyword:
 			p.print("@")
