@@ -1,13 +1,13 @@
 export type Platform = 'browser' | 'node' | 'neutral';
 export type Format = 'iife' | 'cjs' | 'esm';
-export type Loader = 'js' | 'jsx' | 'ts' | 'tsx' | 'css' | 'json' | 'text' | 'base64' | 'file' | 'dataurl' | 'binary' | 'default';
+export type Loader = 'js' | 'jsx' | 'ts' | 'tsx' | 'css' | 'json' | 'text' | 'base64' | 'file' | 'dataurl' | 'binary' | 'copy' | 'default';
 export type LogLevel = 'verbose' | 'debug' | 'info' | 'warning' | 'error' | 'silent';
 export type Charset = 'ascii' | 'utf8';
 export type Drop = 'console' | 'debugger';
 
 interface CommonOptions {
   /** Documentation: https://esbuild.github.io/api/#sourcemap */
-  sourcemap?: boolean | 'inline' | 'external' | 'both';
+  sourcemap?: boolean | 'linked' | 'inline' | 'external' | 'both';
   /** Documentation: https://esbuild.github.io/api/#legal-comments */
   legalComments?: 'none' | 'inline' | 'eof' | 'linked' | 'external';
   /** Documentation: https://esbuild.github.io/api/#source-root */
@@ -17,11 +17,24 @@ interface CommonOptions {
 
   /** Documentation: https://esbuild.github.io/api/#format */
   format?: Format;
-  /** Documentation: https://esbuild.github.io/api/#globalName */
+  /** Documentation: https://esbuild.github.io/api/#global-name */
   globalName?: string;
   /** Documentation: https://esbuild.github.io/api/#target */
   target?: string | string[];
+  /** Documentation: https://esbuild.github.io/api/#supported */
+  supported?: Record<string, boolean>;
+  /** Documentation: https://esbuild.github.io/api/#platform */
+  platform?: Platform;
 
+  /** Documentation: https://esbuild.github.io/api/#mangle-props */
+  mangleProps?: RegExp;
+  /** Documentation: https://esbuild.github.io/api/#mangle-props */
+  reserveProps?: RegExp;
+  /** Documentation: https://esbuild.github.io/api/#mangle-props */
+  mangleQuoted?: boolean;
+  /** Documentation: https://esbuild.github.io/api/#mangle-props */
+  mangleCache?: Record<string, string | false>;
+  /** Documentation: https://esbuild.github.io/api/#drop */
   drop?: Drop[];
   /** Documentation: https://esbuild.github.io/api/#minify */
   minify?: boolean;
@@ -39,11 +52,17 @@ interface CommonOptions {
   ignoreAnnotations?: boolean;
 
   /** Documentation: https://esbuild.github.io/api/#jsx */
-  jsx?: 'transform' | 'preserve';
+  jsx?: 'transform' | 'preserve' | 'automatic';
   /** Documentation: https://esbuild.github.io/api/#jsx-factory */
   jsxFactory?: string;
   /** Documentation: https://esbuild.github.io/api/#jsx-fragment */
   jsxFragment?: string;
+  /** Documentation: https://esbuild.github.io/api/#jsx-import-source */
+  jsxImportSource?: string;
+  /** Documentation: https://esbuild.github.io/api/#jsx-development */
+  jsxDev?: boolean;
+  /** Documentation: https://esbuild.github.io/api/#jsx-side-effects */
+  jsxSideEffects?: boolean;
 
   /** Documentation: https://esbuild.github.io/api/#define */
   define?: { [key: string]: string };
@@ -58,6 +77,8 @@ interface CommonOptions {
   logLevel?: LogLevel;
   /** Documentation: https://esbuild.github.io/api/#log-limit */
   logLimit?: number;
+  /** Documentation: https://esbuild.github.io/api/#log-override */
+  logOverride?: Record<string, LogLevel>;
 }
 
 export interface BuildOptions extends CommonOptions {
@@ -75,15 +96,15 @@ export interface BuildOptions extends CommonOptions {
   outdir?: string;
   /** Documentation: https://esbuild.github.io/api/#outbase */
   outbase?: string;
-  /** Documentation: https://esbuild.github.io/api/#platform */
-  platform?: Platform;
   /** Documentation: https://esbuild.github.io/api/#external */
   external?: string[];
+  /** Documentation: https://esbuild.github.io/api/#alias */
+  alias?: Record<string, string>;
   /** Documentation: https://esbuild.github.io/api/#loader */
   loader?: { [ext: string]: Loader };
   /** Documentation: https://esbuild.github.io/api/#resolve-extensions */
   resolveExtensions?: string[];
-  /** Documentation: https://esbuild.github.io/api/#mainFields */
+  /** Documentation: https://esbuild.github.io/api/#main-fields */
   mainFields?: string[];
   /** Documentation: https://esbuild.github.io/api/#conditions */
   conditions?: string[];
@@ -130,13 +151,14 @@ export interface WatchMode {
 }
 
 export interface StdinOptions {
-  contents: string;
+  contents: string | Uint8Array;
   resolveDir?: string;
   sourcefile?: string;
   loader?: Loader;
 }
 
 export interface Message {
+  id: string;
   pluginName: string;
   text: string;
   location: Location | null;
@@ -195,6 +217,8 @@ export interface BuildResult {
   stop?: () => void;
   /** Only when "metafile: true" */
   metafile?: Metafile;
+  /** Only when "mangleCache" is present */
+  mangleCache?: Record<string, string | false>;
 }
 
 export interface BuildFailure extends Error {
@@ -230,11 +254,15 @@ export interface ServeResult {
 export interface TransformOptions extends CommonOptions {
   tsconfigRaw?: string | {
     compilerOptions?: {
+      alwaysStrict?: boolean,
+      importsNotUsedAsValues?: 'remove' | 'preserve' | 'error',
+      jsx?: 'react' | 'react-jsx' | 'react-jsxdev' | 'preserve',
       jsxFactory?: string,
       jsxFragmentFactory?: string,
-      useDefineForClassFields?: boolean,
-      importsNotUsedAsValues?: 'remove' | 'preserve' | 'error',
+      jsxImportSource?: string,
       preserveValueImports?: boolean,
+      target?: string,
+      useDefineForClassFields?: boolean,
     },
   };
 
@@ -248,6 +276,8 @@ export interface TransformResult {
   code: string;
   map: string;
   warnings: Message[];
+  /** Only when "mangleCache" is present */
+  mangleCache?: Record<string, string | false>;
 }
 
 export interface TransformFailure extends Error {
@@ -387,6 +417,7 @@ export interface OnLoadResult {
 }
 
 export interface PartialMessage {
+  id?: string;
   pluginName?: string;
   text?: string;
   location?: Partial<Location> | null;
@@ -423,6 +454,7 @@ export interface Metafile {
       }[]
       exports: string[]
       entryPoint?: string
+      cssBundle?: string
     }
   }
 }
@@ -476,7 +508,7 @@ export declare function serve(serveOptions: ServeOptions, buildOptions: BuildOpt
  *
  * Documentation: https://esbuild.github.io/api/#transform-api
  */
-export declare function transform(input: string, options?: TransformOptions): Promise<TransformResult>;
+export declare function transform(input: string | Uint8Array, options?: TransformOptions): Promise<TransformResult>;
 
 /**
  * Converts log messages to formatted message strings suitable for printing in
@@ -556,7 +588,17 @@ export interface InitializeOptions {
    * The URL of the "esbuild.wasm" file. This must be provided when running
    * esbuild in the browser.
    */
-  wasmURL?: string
+  wasmURL?: string | URL
+
+  /**
+   * The result of calling "new WebAssembly.Module(buffer)" where "buffer"
+   * is a typed array or ArrayBuffer containing the binary code of the
+   * "esbuild.wasm" file.
+   *
+   * You can use this as an alternative to "wasmURL" for environments where it's
+   * not possible to download the WebAssembly module.
+   */
+  wasmModule?: WebAssembly.Module
 
   /**
    * By default esbuild runs the WebAssembly-based browser API in a web worker
