@@ -602,6 +602,21 @@ func parseOptionsImpl(
 		case strings.HasPrefix(arg, "--inject:") && buildOpts != nil:
 			buildOpts.Inject = append(buildOpts.Inject, arg[len("--inject:"):])
 
+		case strings.HasPrefix(arg, "--alias:") && buildOpts != nil:
+			value := arg[len("--alias:"):]
+			equals := strings.IndexByte(value, '=')
+			if equals == -1 {
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
+					fmt.Sprintf("Missing \"=\" in %q", arg),
+					"You need to use \"=\" to specify both the original package name and the replacement package name. "+
+						"For example, \"--alias:old=new\" replaces package \"old\" with package \"new\".",
+				)
+			}
+			if buildOpts.Alias == nil {
+				buildOpts.Alias = make(map[string]string)
+			}
+			buildOpts.Alias[value[:equals]] = value[equals+1:]
+
 		case strings.HasPrefix(arg, "--jsx="):
 			value := arg[len("--jsx="):]
 			var mode api.JSXMode
@@ -828,6 +843,7 @@ func parseOptionsImpl(
 			}
 
 			colon := map[string]bool{
+				"alias":         true,
 				"banner":        true,
 				"define":        true,
 				"drop":          true,
@@ -1048,20 +1064,16 @@ func runImpl(osArgs []string) int {
 
 	switch {
 	case buildOptions != nil:
-		for _, key := range os.Environ() {
-			// Read the "NODE_PATH" from the environment. This is part of node's
-			// module resolution algorithm. Documentation for this can be found here:
-			// https://nodejs.org/api/modules.html#modules_loading_from_the_global_folders
-			if strings.HasPrefix(key, "NODE_PATH=") {
-				value := key[len("NODE_PATH="):]
-				separator := ":"
-				if fs.CheckIfWindows() {
-					// On Windows, NODE_PATH is delimited by semicolons instead of colons
-					separator = ";"
-				}
-				buildOptions.NodePaths = splitWithEmptyCheck(value, separator)
-				break
+		// Read the "NODE_PATH" from the environment. This is part of node's
+		// module resolution algorithm. Documentation for this can be found here:
+		// https://nodejs.org/api/modules.html#modules_loading_from_the_global_folders
+		if value, ok := os.LookupEnv("NODE_PATH"); ok {
+			separator := ":"
+			if fs.CheckIfWindows() {
+				// On Windows, NODE_PATH is delimited by semicolons instead of colons
+				separator = ";"
 			}
+			buildOptions.NodePaths = splitWithEmptyCheck(value, separator)
 		}
 
 		// Read from stdin when there are no entry points
